# El método de Newton
# para encontrar el punto sobre la elipse
# mas cercano al punto dado en la línea de comandos
#
import sys
import math 


# El valor inicial lo pedimos en la línea de comandos
n = len( sys.argv )
if n != 3 :
	print( "Args: x y" )
	sys.exit(1)

x = float (sys.argv[1] )
y = float (sys.argv[2] )

# La elipse:
a = 7.0
b = 3.0

def f(t,a,b,x,y) :
	c = math.cos(t)
	s = math.sin(t)

	return a*x*s - b*y*c + s*c*(b*b - a*a)

# f(t) = a*x*s - b*y*c + s*c*(b*b - a*a)
# f'(t) = axc - by(-s) + c*c*(b*b - a*a) + (b*b - a*a)(-s)s
# f'(t) = axc + bys + (b*b - a*a)*( c*c - s*s )
def fp(t,a,b,x,y) :
	c = math.cos(t)
	s = math.sin(t)
	return a*x*c + b*y*s + (b*b - a*a)*( c*c - s*s )

# Calculamos el punto incial como en el paper
# Least-squares orthogonal distances "tting of circle,
#     sphere, ellipse, hyperbola, and parabola
# Sung Joon Ahn, Wolfgang Rauh, Hans-Jürgen Warnecke,
# Pattern Recognition 34 (2001) 2283}2303
# x(t) = a cos(t)
# y(t) = b sen(t)
# Voy a obtener el 'x' y 'y' inicial, y entonces
#  cos(t) = x/a
#  sen(t) = y/b
#  t = atan2(y/b, x/a)
#
# De paper

v = (a*b)/math.sqrt( b*b*x*x + a*a*y*y )
xk1 = x*v
yk1 = y*v

if math.fabs(x) < a :
	xk2 = x
	if y < 0 :
		signo = -1.0
	else :
		signo = 1.0
		
	yk2 = signo*(b/a)*math.sqrt( a*a - x*x ) 
else :
	if x < 0 :
		signo = -1.0
	else :
		signo = 1.0

	xk2 = signo * a
	yk2 = 0
	
t = math.atan2(  (yk1+yk2)/(2.0*b),  (xk1+xk2)/(2.0*a)  )

i=1
while i <= 20 :
	d = -f(t,a,b,x,y)/fp(t,a,b,x,y)

	if abs(d) < 1e-8 :
		break

	print( i, t )

	t += d	
	i += 1

x = a * math.cos(t)
y = b * math.sin(t)

print( t, x, y )
